/**
*Buzz Chess Engine
*recog.h
*
*Copyright (C) 2007 Pradu Kannan. All rights reserved.
**/

#ifndef _recogh
#define _recogh

/*
*Dependancies
*/

#include "defs.h"
#include "board.h"
#include "eval.h"

//A recognizer win's minimal score
#define RECOG_WIN (MATE-10000)
#define USE_RECOGDRAW

/*
*Defenitions
*/

//recognizer signatures are of type unsigned int

/*********************Format of a Recognizer Signature*************************
Bit     (1)	0       indicates the existance of white pawns
Bit     (1)	1       indicates the existance of white knights
Bit     (1)	2       indicates the existance of white bishops
Bit     (1)	3       indicates the existance of white rooks
Bit     (1)	4       indicates the existance of white queens
Bit     (1)	5       indicates the existance of black pawns
Bit     (1)	6       indicates the existance of black knights
Bit     (1)	7       indicates the existance of black bishops
Bit     (1)	8       indicates the existance of black rooks
Bit     (1)	9       indicates the existance of black queens
*******************************************************************************/

//Recognizer Signature
#define rsig(msigWHITE,msigBLACK) (((msigBLACK)<<5)|(msigWHITE))

#define extractMatSig(recogsig,side) (side?(recogsig)>>5:(recogsig)&0x1F)

//Material Signature
#define msig(p,n,b,r,q) ( (p?1:0)|((n?1:0)<<1)|((b?1:0)<<2)|((r?1:0)<<3)|((q?1:0)<<4) )
#define msigHasPiece(matsig,piece) ((matsig)&(1<<(piece)))


#define genrsig(pos) rsig(\
	msig(piecesWHITE(pos,P),piecesWHITE(pos,N),piecesWHITE(pos,B),piecesWHITE(pos,R),piecesWHITE(pos,Q))  ,\
	msig(piecesBLACK(pos,P),piecesBLACK(pos,N),piecesBLACK(pos,B),piecesBLACK(pos,R),piecesBLACK(pos,Q))  )

#define updateRecogSig(sig,side,piece,existance) \
{ \
	if(existance) \
		(sig)|=(1<<(piece))<<(5*(side)); \
	else \
		(sig)&=~((1<<(piece))<<(5*(side))); \
}

#define UNRECOGNIZED ILLEGAL_SCORE

//All recognizer signatures
#define KK           0
#define KPK          1
#define KNK          2
#define KBK          4
#define KRK          8
#define KQK          16
#define KKP          32
#define KKN          64
#define KKB          128
#define KKR          256
#define KKQ          512
#define KPNK         3
#define KPBK         5
#define KNBK         6
#define KPRK         9
#define KNRK         10
#define KBRK         12
#define KPQK         17
#define KNQK         18
#define KBQK         20
#define KRQK         24
#define KPKP         33
#define KNKP         34
#define KBKP         36
#define KRKP         40
#define KQKP         48
#define KPKN         65
#define KNKN         66
#define KBKN         68
#define KRKN         72
#define KQKN         80
#define KKPN         96
#define KPKB         129
#define KNKB         130
#define KBKB         132
#define KRKB         136
#define KQKB         144
#define KKPB         160
#define KKNB         192
#define KPKR         257
#define KNKR         258
#define KBKR         260
#define KRKR         264
#define KQKR         272
#define KKPR         288
#define KKNR         320
#define KKBR         384
#define KPKQ         513
#define KNKQ         514
#define KBKQ         516
#define KRKQ         520
#define KQKQ         528
#define KKPQ         544
#define KKNQ         576
#define KKBQ         640
#define KKRQ         768
#define KPNBK        7
#define KPNRK        11
#define KPBRK        13
#define KNBRK        14
#define KPNQK        19
#define KPBQK        21
#define KNBQK        22
#define KPRQK        25
#define KNRQK        26
#define KBRQK        28
#define KPNKP        35
#define KPBKP        37
#define KNBKP        38
#define KPRKP        41
#define KNRKP        42
#define KBRKP        44
#define KPQKP        49
#define KNQKP        50
#define KBQKP        52
#define KRQKP        56
#define KPNKN        67
#define KPBKN        69
#define KNBKN        70
#define KPRKN        73
#define KNRKN        74
#define KBRKN        76
#define KPQKN        81
#define KNQKN        82
#define KBQKN        84
#define KRQKN        88
#define KPKPN        97
#define KNKPN        98
#define KBKPN        100
#define KRKPN        104
#define KQKPN        112
#define KPNKB        131
#define KPBKB        133
#define KNBKB        134
#define KPRKB        137
#define KNRKB        138
#define KBRKB        140
#define KPQKB        145
#define KNQKB        146
#define KBQKB        148
#define KRQKB        152
#define KPKPB        161
#define KNKPB        162
#define KBKPB        164
#define KRKPB        168
#define KQKPB        176
#define KPKNB        193
#define KNKNB        194
#define KBKNB        196
#define KRKNB        200
#define KQKNB        208
#define KKPNB        224
#define KPNKR        259
#define KPBKR        261
#define KNBKR        262
#define KPRKR        265
#define KNRKR        266
#define KBRKR        268
#define KPQKR        273
#define KNQKR        274
#define KBQKR        276
#define KRQKR        280
#define KPKPR        289
#define KNKPR        290
#define KBKPR        292
#define KRKPR        296
#define KQKPR        304
#define KPKNR        321
#define KNKNR        322
#define KBKNR        324
#define KRKNR        328
#define KQKNR        336
#define KKPNR        352
#define KPKBR        385
#define KNKBR        386
#define KBKBR        388
#define KRKBR        392
#define KQKBR        400
#define KKPBR        416
#define KKNBR        448
#define KPNKQ        515
#define KPBKQ        517
#define KNBKQ        518
#define KPRKQ        521
#define KNRKQ        522
#define KBRKQ        524
#define KPQKQ        529
#define KNQKQ        530
#define KBQKQ        532
#define KRQKQ        536
#define KPKPQ        545
#define KNKPQ        546
#define KBKPQ        548
#define KRKPQ        552
#define KQKPQ        560
#define KPKNQ        577
#define KNKNQ        578
#define KBKNQ        580
#define KRKNQ        584
#define KQKNQ        592
#define KKPNQ        608
#define KPKBQ        641
#define KNKBQ        642
#define KBKBQ        644
#define KRKBQ        648
#define KQKBQ        656
#define KKPBQ        672
#define KKNBQ        704
#define KPKRQ        769
#define KNKRQ        770
#define KBKRQ        772
#define KRKRQ        776
#define KQKRQ        784
#define KKPRQ        800
#define KKNRQ        832
#define KKBRQ        896
#define KPNBRK       15
#define KPNBQK       23
#define KPNRQK       27
#define KPBRQK       29
#define KNBRQK       30
#define KPNBKP       39
#define KPNRKP       43
#define KPBRKP       45
#define KNBRKP       46
#define KPNQKP       51
#define KPBQKP       53
#define KNBQKP       54
#define KPRQKP       57
#define KNRQKP       58
#define KBRQKP       60
#define KPNBKN       71
#define KPNRKN       75
#define KPBRKN       77
#define KNBRKN       78
#define KPNQKN       83
#define KPBQKN       85
#define KNBQKN       86
#define KPRQKN       89
#define KNRQKN       90
#define KBRQKN       92
#define KPNKPN       99
#define KPBKPN       101
#define KNBKPN       102
#define KPRKPN       105
#define KNRKPN       106
#define KBRKPN       108
#define KPQKPN       113
#define KNQKPN       114
#define KBQKPN       116
#define KRQKPN       120
#define KPNBKB       135
#define KPNRKB       139
#define KPBRKB       141
#define KNBRKB       142
#define KPNQKB       147
#define KPBQKB       149
#define KNBQKB       150
#define KPRQKB       153
#define KNRQKB       154
#define KBRQKB       156
#define KPNKPB       163
#define KPBKPB       165
#define KNBKPB       166
#define KPRKPB       169
#define KNRKPB       170
#define KBRKPB       172
#define KPQKPB       177
#define KNQKPB       178
#define KBQKPB       180
#define KRQKPB       184
#define KPNKNB       195
#define KPBKNB       197
#define KNBKNB       198
#define KPRKNB       201
#define KNRKNB       202
#define KBRKNB       204
#define KPQKNB       209
#define KNQKNB       210
#define KBQKNB       212
#define KRQKNB       216
#define KPKPNB       225
#define KNKPNB       226
#define KBKPNB       228
#define KRKPNB       232
#define KQKPNB       240
#define KPNBKR       263
#define KPNRKR       267
#define KPBRKR       269
#define KNBRKR       270
#define KPNQKR       275
#define KPBQKR       277
#define KNBQKR       278
#define KPRQKR       281
#define KNRQKR       282
#define KBRQKR       284
#define KPNKPR       291
#define KPBKPR       293
#define KNBKPR       294
#define KPRKPR       297
#define KNRKPR       298
#define KBRKPR       300
#define KPQKPR       305
#define KNQKPR       306
#define KBQKPR       308
#define KRQKPR       312
#define KPNKNR       323
#define KPBKNR       325
#define KNBKNR       326
#define KPRKNR       329
#define KNRKNR       330
#define KBRKNR       332
#define KPQKNR       337
#define KNQKNR       338
#define KBQKNR       340
#define KRQKNR       344
#define KPKPNR       353
#define KNKPNR       354
#define KBKPNR       356
#define KRKPNR       360
#define KQKPNR       368
#define KPNKBR       387
#define KPBKBR       389
#define KNBKBR       390
#define KPRKBR       393
#define KNRKBR       394
#define KBRKBR       396
#define KPQKBR       401
#define KNQKBR       402
#define KBQKBR       404
#define KRQKBR       408
#define KPKPBR       417
#define KNKPBR       418
#define KBKPBR       420
#define KRKPBR       424
#define KQKPBR       432
#define KPKNBR       449
#define KNKNBR       450
#define KBKNBR       452
#define KRKNBR       456
#define KQKNBR       464
#define KKPNBR       480
#define KPNBKQ       519
#define KPNRKQ       523
#define KPBRKQ       525
#define KNBRKQ       526
#define KPNQKQ       531
#define KPBQKQ       533
#define KNBQKQ       534
#define KPRQKQ       537
#define KNRQKQ       538
#define KBRQKQ       540
#define KPNKPQ       547
#define KPBKPQ       549
#define KNBKPQ       550
#define KPRKPQ       553
#define KNRKPQ       554
#define KBRKPQ       556
#define KPQKPQ       561
#define KNQKPQ       562
#define KBQKPQ       564
#define KRQKPQ       568
#define KPNKNQ       579
#define KPBKNQ       581
#define KNBKNQ       582
#define KPRKNQ       585
#define KNRKNQ       586
#define KBRKNQ       588
#define KPQKNQ       593
#define KNQKNQ       594
#define KBQKNQ       596
#define KRQKNQ       600
#define KPKPNQ       609
#define KNKPNQ       610
#define KBKPNQ       612
#define KRKPNQ       616
#define KQKPNQ       624
#define KPNKBQ       643
#define KPBKBQ       645
#define KNBKBQ       646
#define KPRKBQ       649
#define KNRKBQ       650
#define KBRKBQ       652
#define KPQKBQ       657
#define KNQKBQ       658
#define KBQKBQ       660
#define KRQKBQ       664
#define KPKPBQ       673
#define KNKPBQ       674
#define KBKPBQ       676
#define KRKPBQ       680
#define KQKPBQ       688
#define KPKNBQ       705
#define KNKNBQ       706
#define KBKNBQ       708
#define KRKNBQ       712
#define KQKNBQ       720
#define KKPNBQ       736
#define KPNKRQ       771
#define KPBKRQ       773
#define KNBKRQ       774
#define KPRKRQ       777
#define KNRKRQ       778
#define KBRKRQ       780
#define KPQKRQ       785
#define KNQKRQ       786
#define KBQKRQ       788
#define KRQKRQ       792
#define KPKPRQ       801
#define KNKPRQ       802
#define KBKPRQ       804
#define KRKPRQ       808
#define KQKPRQ       816
#define KPKNRQ       833
#define KNKNRQ       834
#define KBKNRQ       836
#define KRKNRQ       840
#define KQKNRQ       848
#define KKPNRQ       864
#define KPKBRQ       897
#define KNKBRQ       898
#define KBKBRQ       900
#define KRKBRQ       904
#define KQKBRQ       912
#define KKPBRQ       928
#define KKNBRQ       960
#define KPNBRQK      31
#define KPNBRKP      47
#define KPNBQKP      55
#define KPNRQKP      59
#define KPBRQKP      61
#define KNBRQKP      62
#define KPNBRKN      79
#define KPNBQKN      87
#define KPNRQKN      91
#define KPBRQKN      93
#define KNBRQKN      94
#define KPNBKPN      103
#define KPNRKPN      107
#define KPBRKPN      109
#define KNBRKPN      110
#define KPNQKPN      115
#define KPBQKPN      117
#define KNBQKPN      118
#define KPRQKPN      121
#define KNRQKPN      122
#define KBRQKPN      124
#define KPNBRKB      143
#define KPNBQKB      151
#define KPNRQKB      155
#define KPBRQKB      157
#define KNBRQKB      158
#define KPNBKPB      167
#define KPNRKPB      171
#define KPBRKPB      173
#define KNBRKPB      174
#define KPNQKPB      179
#define KPBQKPB      181
#define KNBQKPB      182
#define KPRQKPB      185
#define KNRQKPB      186
#define KBRQKPB      188
#define KPNBKNB      199
#define KPNRKNB      203
#define KPBRKNB      205
#define KNBRKNB      206
#define KPNQKNB      211
#define KPBQKNB      213
#define KNBQKNB      214
#define KPRQKNB      217
#define KNRQKNB      218
#define KBRQKNB      220
#define KPNKPNB      227
#define KPBKPNB      229
#define KNBKPNB      230
#define KPRKPNB      233
#define KNRKPNB      234
#define KBRKPNB      236
#define KPQKPNB      241
#define KNQKPNB      242
#define KBQKPNB      244
#define KRQKPNB      248
#define KPNBRKR      271
#define KPNBQKR      279
#define KPNRQKR      283
#define KPBRQKR      285
#define KNBRQKR      286
#define KPNBKPR      295
#define KPNRKPR      299
#define KPBRKPR      301
#define KNBRKPR      302
#define KPNQKPR      307
#define KPBQKPR      309
#define KNBQKPR      310
#define KPRQKPR      313
#define KNRQKPR      314
#define KBRQKPR      316
#define KPNBKNR      327
#define KPNRKNR      331
#define KPBRKNR      333
#define KNBRKNR      334
#define KPNQKNR      339
#define KPBQKNR      341
#define KNBQKNR      342
#define KPRQKNR      345
#define KNRQKNR      346
#define KBRQKNR      348
#define KPNKPNR      355
#define KPBKPNR      357
#define KNBKPNR      358
#define KPRKPNR      361
#define KNRKPNR      362
#define KBRKPNR      364
#define KPQKPNR      369
#define KNQKPNR      370
#define KBQKPNR      372
#define KRQKPNR      376
#define KPNBKBR      391
#define KPNRKBR      395
#define KPBRKBR      397
#define KNBRKBR      398
#define KPNQKBR      403
#define KPBQKBR      405
#define KNBQKBR      406
#define KPRQKBR      409
#define KNRQKBR      410
#define KBRQKBR      412
#define KPNKPBR      419
#define KPBKPBR      421
#define KNBKPBR      422
#define KPRKPBR      425
#define KNRKPBR      426
#define KBRKPBR      428
#define KPQKPBR      433
#define KNQKPBR      434
#define KBQKPBR      436
#define KRQKPBR      440
#define KPNKNBR      451
#define KPBKNBR      453
#define KNBKNBR      454
#define KPRKNBR      457
#define KNRKNBR      458
#define KBRKNBR      460
#define KPQKNBR      465
#define KNQKNBR      466
#define KBQKNBR      468
#define KRQKNBR      472
#define KPKPNBR      481
#define KNKPNBR      482
#define KBKPNBR      484
#define KRKPNBR      488
#define KQKPNBR      496
#define KPNBRKQ      527
#define KPNBQKQ      535
#define KPNRQKQ      539
#define KPBRQKQ      541
#define KNBRQKQ      542
#define KPNBKPQ      551
#define KPNRKPQ      555
#define KPBRKPQ      557
#define KNBRKPQ      558
#define KPNQKPQ      563
#define KPBQKPQ      565
#define KNBQKPQ      566
#define KPRQKPQ      569
#define KNRQKPQ      570
#define KBRQKPQ      572
#define KPNBKNQ      583
#define KPNRKNQ      587
#define KPBRKNQ      589
#define KNBRKNQ      590
#define KPNQKNQ      595
#define KPBQKNQ      597
#define KNBQKNQ      598
#define KPRQKNQ      601
#define KNRQKNQ      602
#define KBRQKNQ      604
#define KPNKPNQ      611
#define KPBKPNQ      613
#define KNBKPNQ      614
#define KPRKPNQ      617
#define KNRKPNQ      618
#define KBRKPNQ      620
#define KPQKPNQ      625
#define KNQKPNQ      626
#define KBQKPNQ      628
#define KRQKPNQ      632
#define KPNBKBQ      647
#define KPNRKBQ      651
#define KPBRKBQ      653
#define KNBRKBQ      654
#define KPNQKBQ      659
#define KPBQKBQ      661
#define KNBQKBQ      662
#define KPRQKBQ      665
#define KNRQKBQ      666
#define KBRQKBQ      668
#define KPNKPBQ      675
#define KPBKPBQ      677
#define KNBKPBQ      678
#define KPRKPBQ      681
#define KNRKPBQ      682
#define KBRKPBQ      684
#define KPQKPBQ      689
#define KNQKPBQ      690
#define KBQKPBQ      692
#define KRQKPBQ      696
#define KPNKNBQ      707
#define KPBKNBQ      709
#define KNBKNBQ      710
#define KPRKNBQ      713
#define KNRKNBQ      714
#define KBRKNBQ      716
#define KPQKNBQ      721
#define KNQKNBQ      722
#define KBQKNBQ      724
#define KRQKNBQ      728
#define KPKPNBQ      737
#define KNKPNBQ      738
#define KBKPNBQ      740
#define KRKPNBQ      744
#define KQKPNBQ      752
#define KPNBKRQ      775
#define KPNRKRQ      779
#define KPBRKRQ      781
#define KNBRKRQ      782
#define KPNQKRQ      787
#define KPBQKRQ      789
#define KNBQKRQ      790
#define KPRQKRQ      793
#define KNRQKRQ      794
#define KBRQKRQ      796
#define KPNKPRQ      803
#define KPBKPRQ      805
#define KNBKPRQ      806
#define KPRKPRQ      809
#define KNRKPRQ      810
#define KBRKPRQ      812
#define KPQKPRQ      817
#define KNQKPRQ      818
#define KBQKPRQ      820
#define KRQKPRQ      824
#define KPNKNRQ      835
#define KPBKNRQ      837
#define KNBKNRQ      838
#define KPRKNRQ      841
#define KNRKNRQ      842
#define KBRKNRQ      844
#define KPQKNRQ      849
#define KNQKNRQ      850
#define KBQKNRQ      852
#define KRQKNRQ      856
#define KPKPNRQ      865
#define KNKPNRQ      866
#define KBKPNRQ      868
#define KRKPNRQ      872
#define KQKPNRQ      880
#define KPNKBRQ      899
#define KPBKBRQ      901
#define KNBKBRQ      902
#define KPRKBRQ      905
#define KNRKBRQ      906
#define KBRKBRQ      908
#define KPQKBRQ      913
#define KNQKBRQ      914
#define KBQKBRQ      916
#define KRQKBRQ      920
#define KPKPBRQ      929
#define KNKPBRQ      930
#define KBKPBRQ      932
#define KRKPBRQ      936
#define KQKPBRQ      944
#define KPKNBRQ      961
#define KNKNBRQ      962
#define KBKNBRQ      964
#define KRKNBRQ      968
#define KQKNBRQ      976
#define KKPNBRQ      992
#define KPNBRQKP     63
#define KPNBRQKN     95
#define KPNBRKPN     111
#define KPNBQKPN     119
#define KPNRQKPN     123
#define KPBRQKPN     125
#define KNBRQKPN     126
#define KPNBRQKB     159
#define KPNBRKPB     175
#define KPNBQKPB     183
#define KPNRQKPB     187
#define KPBRQKPB     189
#define KNBRQKPB     190
#define KPNBRKNB     207
#define KPNBQKNB     215
#define KPNRQKNB     219
#define KPBRQKNB     221
#define KNBRQKNB     222
#define KPNBKPNB     231
#define KPNRKPNB     235
#define KPBRKPNB     237
#define KNBRKPNB     238
#define KPNQKPNB     243
#define KPBQKPNB     245
#define KNBQKPNB     246
#define KPRQKPNB     249
#define KNRQKPNB     250
#define KBRQKPNB     252
#define KPNBRQKR     287
#define KPNBRKPR     303
#define KPNBQKPR     311
#define KPNRQKPR     315
#define KPBRQKPR     317
#define KNBRQKPR     318
#define KPNBRKNR     335
#define KPNBQKNR     343
#define KPNRQKNR     347
#define KPBRQKNR     349
#define KNBRQKNR     350
#define KPNBKPNR     359
#define KPNRKPNR     363
#define KPBRKPNR     365
#define KNBRKPNR     366
#define KPNQKPNR     371
#define KPBQKPNR     373
#define KNBQKPNR     374
#define KPRQKPNR     377
#define KNRQKPNR     378
#define KBRQKPNR     380
#define KPNBRKBR     399
#define KPNBQKBR     407
#define KPNRQKBR     411
#define KPBRQKBR     413
#define KNBRQKBR     414
#define KPNBKPBR     423
#define KPNRKPBR     427
#define KPBRKPBR     429
#define KNBRKPBR     430
#define KPNQKPBR     435
#define KPBQKPBR     437
#define KNBQKPBR     438
#define KPRQKPBR     441
#define KNRQKPBR     442
#define KBRQKPBR     444
#define KPNBKNBR     455
#define KPNRKNBR     459
#define KPBRKNBR     461
#define KNBRKNBR     462
#define KPNQKNBR     467
#define KPBQKNBR     469
#define KNBQKNBR     470
#define KPRQKNBR     473
#define KNRQKNBR     474
#define KBRQKNBR     476
#define KPNKPNBR     483
#define KPBKPNBR     485
#define KNBKPNBR     486
#define KPRKPNBR     489
#define KNRKPNBR     490
#define KBRKPNBR     492
#define KPQKPNBR     497
#define KNQKPNBR     498
#define KBQKPNBR     500
#define KRQKPNBR     504
#define KPNBRQKQ     543
#define KPNBRKPQ     559
#define KPNBQKPQ     567
#define KPNRQKPQ     571
#define KPBRQKPQ     573
#define KNBRQKPQ     574
#define KPNBRKNQ     591
#define KPNBQKNQ     599
#define KPNRQKNQ     603
#define KPBRQKNQ     605
#define KNBRQKNQ     606
#define KPNBKPNQ     615
#define KPNRKPNQ     619
#define KPBRKPNQ     621
#define KNBRKPNQ     622
#define KPNQKPNQ     627
#define KPBQKPNQ     629
#define KNBQKPNQ     630
#define KPRQKPNQ     633
#define KNRQKPNQ     634
#define KBRQKPNQ     636
#define KPNBRKBQ     655
#define KPNBQKBQ     663
#define KPNRQKBQ     667
#define KPBRQKBQ     669
#define KNBRQKBQ     670
#define KPNBKPBQ     679
#define KPNRKPBQ     683
#define KPBRKPBQ     685
#define KNBRKPBQ     686
#define KPNQKPBQ     691
#define KPBQKPBQ     693
#define KNBQKPBQ     694
#define KPRQKPBQ     697
#define KNRQKPBQ     698
#define KBRQKPBQ     700
#define KPNBKNBQ     711
#define KPNRKNBQ     715
#define KPBRKNBQ     717
#define KNBRKNBQ     718
#define KPNQKNBQ     723
#define KPBQKNBQ     725
#define KNBQKNBQ     726
#define KPRQKNBQ     729
#define KNRQKNBQ     730
#define KBRQKNBQ     732
#define KPNKPNBQ     739
#define KPBKPNBQ     741
#define KNBKPNBQ     742
#define KPRKPNBQ     745
#define KNRKPNBQ     746
#define KBRKPNBQ     748
#define KPQKPNBQ     753
#define KNQKPNBQ     754
#define KBQKPNBQ     756
#define KRQKPNBQ     760
#define KPNBRKRQ     783
#define KPNBQKRQ     791
#define KPNRQKRQ     795
#define KPBRQKRQ     797
#define KNBRQKRQ     798
#define KPNBKPRQ     807
#define KPNRKPRQ     811
#define KPBRKPRQ     813
#define KNBRKPRQ     814
#define KPNQKPRQ     819
#define KPBQKPRQ     821
#define KNBQKPRQ     822
#define KPRQKPRQ     825
#define KNRQKPRQ     826
#define KBRQKPRQ     828
#define KPNBKNRQ     839
#define KPNRKNRQ     843
#define KPBRKNRQ     845
#define KNBRKNRQ     846
#define KPNQKNRQ     851
#define KPBQKNRQ     853
#define KNBQKNRQ     854
#define KPRQKNRQ     857
#define KNRQKNRQ     858
#define KBRQKNRQ     860
#define KPNKPNRQ     867
#define KPBKPNRQ     869
#define KNBKPNRQ     870
#define KPRKPNRQ     873
#define KNRKPNRQ     874
#define KBRKPNRQ     876
#define KPQKPNRQ     881
#define KNQKPNRQ     882
#define KBQKPNRQ     884
#define KRQKPNRQ     888
#define KPNBKBRQ     903
#define KPNRKBRQ     907
#define KPBRKBRQ     909
#define KNBRKBRQ     910
#define KPNQKBRQ     915
#define KPBQKBRQ     917
#define KNBQKBRQ     918
#define KPRQKBRQ     921
#define KNRQKBRQ     922
#define KBRQKBRQ     924
#define KPNKPBRQ     931
#define KPBKPBRQ     933
#define KNBKPBRQ     934
#define KPRKPBRQ     937
#define KNRKPBRQ     938
#define KBRKPBRQ     940
#define KPQKPBRQ     945
#define KNQKPBRQ     946
#define KBQKPBRQ     948
#define KRQKPBRQ     952
#define KPNKNBRQ     963
#define KPBKNBRQ     965
#define KNBKNBRQ     966
#define KPRKNBRQ     969
#define KNRKNBRQ     970
#define KBRKNBRQ     972
#define KPQKNBRQ     977
#define KNQKNBRQ     978
#define KBQKNBRQ     980
#define KRQKNBRQ     984
#define KPKPNBRQ     993
#define KNKPNBRQ     994
#define KBKPNBRQ     996
#define KRKPNBRQ     1000
#define KQKPNBRQ     1008
#define KPNBRQKPN    127
#define KPNBRQKPB    191
#define KPNBRQKNB    223
#define KPNBRKPNB    239
#define KPNBQKPNB    247
#define KPNRQKPNB    251
#define KPBRQKPNB    253
#define KNBRQKPNB    254
#define KPNBRQKPR    319
#define KPNBRQKNR    351
#define KPNBRKPNR    367
#define KPNBQKPNR    375
#define KPNRQKPNR    379
#define KPBRQKPNR    381
#define KNBRQKPNR    382
#define KPNBRQKBR    415
#define KPNBRKPBR    431
#define KPNBQKPBR    439
#define KPNRQKPBR    443
#define KPBRQKPBR    445
#define KNBRQKPBR    446
#define KPNBRKNBR    463
#define KPNBQKNBR    471
#define KPNRQKNBR    475
#define KPBRQKNBR    477
#define KNBRQKNBR    478
#define KPNBKPNBR    487
#define KPNRKPNBR    491
#define KPBRKPNBR    493
#define KNBRKPNBR    494
#define KPNQKPNBR    499
#define KPBQKPNBR    501
#define KNBQKPNBR    502
#define KPRQKPNBR    505
#define KNRQKPNBR    506
#define KBRQKPNBR    508
#define KPNBRQKPQ    575
#define KPNBRQKNQ    607
#define KPNBRKPNQ    623
#define KPNBQKPNQ    631
#define KPNRQKPNQ    635
#define KPBRQKPNQ    637
#define KNBRQKPNQ    638
#define KPNBRQKBQ    671
#define KPNBRKPBQ    687
#define KPNBQKPBQ    695
#define KPNRQKPBQ    699
#define KPBRQKPBQ    701
#define KNBRQKPBQ    702
#define KPNBRKNBQ    719
#define KPNBQKNBQ    727
#define KPNRQKNBQ    731
#define KPBRQKNBQ    733
#define KNBRQKNBQ    734
#define KPNBKPNBQ    743
#define KPNRKPNBQ    747
#define KPBRKPNBQ    749
#define KNBRKPNBQ    750
#define KPNQKPNBQ    755
#define KPBQKPNBQ    757
#define KNBQKPNBQ    758
#define KPRQKPNBQ    761
#define KNRQKPNBQ    762
#define KBRQKPNBQ    764
#define KPNBRQKRQ    799
#define KPNBRKPRQ    815
#define KPNBQKPRQ    823
#define KPNRQKPRQ    827
#define KPBRQKPRQ    829
#define KNBRQKPRQ    830
#define KPNBRKNRQ    847
#define KPNBQKNRQ    855
#define KPNRQKNRQ    859
#define KPBRQKNRQ    861
#define KNBRQKNRQ    862
#define KPNBKPNRQ    871
#define KPNRKPNRQ    875
#define KPBRKPNRQ    877
#define KNBRKPNRQ    878
#define KPNQKPNRQ    883
#define KPBQKPNRQ    885
#define KNBQKPNRQ    886
#define KPRQKPNRQ    889
#define KNRQKPNRQ    890
#define KBRQKPNRQ    892
#define KPNBRKBRQ    911
#define KPNBQKBRQ    919
#define KPNRQKBRQ    923
#define KPBRQKBRQ    925
#define KNBRQKBRQ    926
#define KPNBKPBRQ    935
#define KPNRKPBRQ    939
#define KPBRKPBRQ    941
#define KNBRKPBRQ    942
#define KPNQKPBRQ    947
#define KPBQKPBRQ    949
#define KNBQKPBRQ    950
#define KPRQKPBRQ    953
#define KNRQKPBRQ    954
#define KBRQKPBRQ    956
#define KPNBKNBRQ    967
#define KPNRKNBRQ    971
#define KPBRKNBRQ    973
#define KNBRKNBRQ    974
#define KPNQKNBRQ    979
#define KPBQKNBRQ    981
#define KNBQKNBRQ    982
#define KPRQKNBRQ    985
#define KNRQKNBRQ    986
#define KBRQKNBRQ    988
#define KPNKPNBRQ    995
#define KPBKPNBRQ    997
#define KNBKPNBRQ    998
#define KPRKPNBRQ    1001
#define KNRKPNBRQ    1002
#define KBRKPNBRQ    1004
#define KPQKPNBRQ    1009
#define KNQKPNBRQ    1010
#define KBQKPNBRQ    1012
#define KRQKPNBRQ    1016
#define KPNBRQKPNB   255
#define KPNBRQKPNR   383
#define KPNBRQKPBR   447
#define KPNBRQKNBR   479
#define KPNBRKPNBR   495
#define KPNBQKPNBR   503
#define KPNRQKPNBR   507
#define KPBRQKPNBR   509
#define KNBRQKPNBR   510
#define KPNBRQKPNQ   639
#define KPNBRQKPBQ   703
#define KPNBRQKNBQ   735
#define KPNBRKPNBQ   751
#define KPNBQKPNBQ   759
#define KPNRQKPNBQ   763
#define KPBRQKPNBQ   765
#define KNBRQKPNBQ   766
#define KPNBRQKPRQ   831
#define KPNBRQKNRQ   863
#define KPNBRKPNRQ   879
#define KPNBQKPNRQ   887
#define KPNRQKPNRQ   891
#define KPBRQKPNRQ   893
#define KNBRQKPNRQ   894
#define KPNBRQKBRQ   927
#define KPNBRKPBRQ   943
#define KPNBQKPBRQ   951
#define KPNRQKPBRQ   955
#define KPBRQKPBRQ   957
#define KNBRQKPBRQ   958
#define KPNBRKNBRQ   975
#define KPNBQKNBRQ   983
#define KPNRQKNBRQ   987
#define KPBRQKNBRQ   989
#define KNBRQKNBRQ   990
#define KPNBKPNBRQ   999
#define KPNRKPNBRQ   1003
#define KPBRKPNBRQ   1005
#define KNBRKPNBRQ   1006
#define KPNQKPNBRQ   1011
#define KPBQKPNBRQ   1013
#define KNBQKPNBRQ   1014
#define KPRQKPNBRQ   1017
#define KNRQKPNBRQ   1018
#define KBRQKPNBRQ   1020
#define KPNBRQKPNBR  511
#define KPNBRQKPNBQ  767
#define KPNBRQKPNRQ  895
#define KPNBRQKPBRQ  959
#define KPNBRQKNBRQ  991
#define KPNBRKPNBRQ  1007
#define KPNBQKPNBRQ  1015
#define KPNRQKPNBRQ  1019
#define KPBRQKPNBRQ  1021
#define KNBRQKPNBRQ  1022
#define KPNBRQKPNBRQ 1023

/*
*Function Prototypes
*/

bool recogDraw(const board* pos);

#endif //_recogh
